#include <stdio.h>
#define UINT unsigned int
#define UINTMAX 4294967295

#define E_EXISTE 1
#define E_INDICE_GRAPH 2
#define E_POID 3
#define E_CREATION_GRAPH 4
#define E_MEMOIRE_GRAPH 5

class graphe_p
{
	private:
	int n;
	UINT **graph;
	
	public:
        graphe_p(int _n);

	int taille();
	void ajouter(int x,int y,UINT poid);
	char existe(int x,int y);
	UINT cout(int x,int y);

	private:
        void erreur(int n);
};

graphe_p :: graphe_p(int _n)
{
	int x,y;

	if (n < 0)
		erreur(E_CREATION_GRAPH);

	n = _n;

        if (!(graph = (UINT **) calloc(n + 1,sizeof(UINT *))))
		erreur(E_MEMOIRE_GRAPH);
	for (x = 1;x <= n;x++)
        {
		graph[x] = (UINT *) calloc(n + 1,sizeof(UINT));
		for (y = 1;y <= n;y++)
			graph[x][y] = UINTMAX;
	}
}

int graphe_p :: taille()
{
	return(n);
}

void graphe_p :: ajouter(int x,int y,UINT poid)
{
	if ((x < 1) || (x > n) || (y < 1) || (y > n))
		erreur(E_INDICE_GRAPH);
		
	if (poid >= UINTMAX)
		erreur(E_POID);
	
	graph[x][y] = poid;
}

char graphe_p :: existe(int x,int y)
{
	if ((x < 1) || (x > n) || (y < 1) || (y > n))
		erreur(E_INDICE_GRAPH);
		
	return((char) !(graph[x][y] == UINTMAX));
}

UINT graphe_p :: cout(int x,int y)
{
	if ((x < 1) || (x > n) || (y < 1) || (y > n))
		erreur(E_INDICE_GRAPH);
		
	if (!existe(x,y))
		erreur(E_EXISTE);	
	
	return(graph[x][y]);	
}

void graphe_p :: erreur(int n)
{
	switch(n)
	{
		case E_EXISTE: printf("Erreur : cout d'un arc inexistant\n"); break;
		case E_INDICE_GRAPH: printf("Erreur : indices inexitant dans le graphe\n"); break;
		case E_POID: printf("Erreur : ajout d'un arc ayant un poid trop eleve"); break;
		case E_CREATION_GRAPH: printf("Erreur : nombre de noeud d'un graphe"); break;
		case E_MEMOIRE_GRAPH: printf("Erreur : memoire pour un graphe"); break;
	}
	exit(0);
}
