#include <stdio.h>
#include "trans_sol.H"

#define E_TAILLE 1
#define E_NON_ARBRE 2
#define E_MEMOIRE 3
#define E_SOMME 4

class transhipment
{
	private:
	int n;
	graphe_p *graph;
	int *noeuds;

	public:
	transhipment(int _n,graphe_p *_graph,int *_noeuds);

	private:
	void initialisation(graphe_p *graph_ini,arbre_p *arbre_ini);
	int trouve_sous_racine(int *sous_racines,arbre_p *arbre_ini);
	public:
	trans_sol *resolution();

	private:
	void erreur(int k);
};

transhipment :: transhipment(int _n,graphe_p *_graph,int *_noeuds)
{
	int k,somme = 0;

	if (_n != _graph->taille())
		erreur(E_TAILLE);

	n = _n;
	graph = _graph;

	for(k = 1;k <= n;k++)
		somme += _noeuds[k];
	
	if (somme)
		erreur(E_SOMME);

	noeuds = _noeuds;
}

void transhipment :: initialisation(graphe_p *graph_ini,arbre_p *arbre_ini)
{
	int x,y,k;
	trans_ini *t;

	printf("********************* Initialisation *********************\n");
/*	Cration du graphe pour l'initialisation	*/
/*	Copie des arcs de graphe	*/
	for(y = 1;y <= n;y++)
		for(x = 1;x <= n;x++)
			if (graph->existe(x,y))
				graph_ini->ajouter(x,y,0);
/*	Ajout des arcs fictifs	*/
	for(k = 2;k <= n;k++)
	{
		if(arbre_ini->sens(k) && !graph->existe(1,k))
			graph_ini->ajouter(1,k,1);
		if(!arbre_ini->sens(k) && !graph->existe(k,1))
			graph_ini->ajouter(k,1,1);
	}
	
/*	Resolution de ce probleme de transport	*/
	t = new trans_ini(n,graph_ini,arbre_ini);
	t->resolution();
}

int transhipment :: trouve_sous_racine(int *sous_racines,arbre_p *arbre_ini)
{
	int a,nb = 1;

	sous_racines[0] = 1;
	a = arbre_ini->s(1);
	
	while(a != 1)
	{
		if(arbre_ini->sens(a))
		{
			if(!graph->existe(arbre_ini->p(a),a))
			{
				if(arbre_ini->poid(a))
					return(0);	//Il n'y pas de solution
				else
				{
					sous_racines[nb] = a;
					nb++;
				}
			}
		}
		else
		{
			if(!graph->existe(a,arbre_ini->p(a)))
			{
				if(arbre_ini->poid(a))
					return(0);	//Il n'y pas de solution
				else
				{
					sous_racines[nb] = a;
					nb++;
				}
			}
		}
		a = arbre_ini->s(a);
	}
	return(nb);
}

trans_sol *transhipment :: resolution()
{
	int nb,k;
	arbre_p *arbre_ini,*arbre;
	graphe_p *graph_ini;
	trans_ini *t;
	trans_sol *solution;
	int *sous_racines;
	
	arbre_ini = new arbre_p(n,noeuds);
	graph_ini = new graphe_p(n);
/*	Calcul de l'arbre d'initialisation	*/
	initialisation(graph_ini,arbre_ini);
	
/*	Analyse de l'arbre d'initilisation	*/
	printf("************** Analyse de l'initialisation ***************\n");
	
	if (!(sous_racines = (int *) calloc(n,sizeof(int))))
		erreur(E_MEMOIRE);
	
	if ((nb = trouve_sous_racine(sous_racines,arbre_ini)) > 0)
	{
		printf("Le probleme admet une solution\n");
		if (nb > 1)
			printf("Le probleme se decompose en %d problemes de transport\n",nb);
		printf("\n");
		
		solution = new trans_sol(nb);
		
/*	Resolution du probleme pour chaque sous-arbre	*/
		k = 0;
		do
		{
			printf("**** Resolution du probleme pour l'arbre de racine %d *****\n",sous_racines[k]);
			arbre = new arbre_p(n,sous_racines[k],graph,arbre_ini);
			t = new trans_ini(n,graph,arbre);
			t->resolution();
			solution->definir(k,arbre);
		
			k++;
		}
		while(k != nb);
		
		return(solution);
	}
	else
	{
		printf("Le probleme n'admet pas de solution\n\n");
		return(new trans_sol(0));
	}
}
		
void transhipment :: erreur(int k)
{
	switch(k)
	{
		case E_TAILLE: printf("Erreur : taille incoherante\n"); break;
		case E_NON_ARBRE: printf("Erreur : l'arbre_rec n'est pas un arbre"); break;
		case E_MEMOIRE: printf("Erreur : memoire\n"); break;
		case E_SOMME: printf("Erreur : la somme des noeuds est non nul\n"); 
	}
	exit(0);
}
